# Copyright 2021 (c) Anna Schumaker.
import datetime
import db
import pathlib
import sqlite3
import unittest
from gi.repository import GObject

class TestTrack(unittest.TestCase):
    def setUp(self):
        db.reset()

    def test_init(self):
        track = db.make_fake_track(1, 1.234, "Test Title", "/a/b/c/1.ogg")
        self.assertIsInstance(track.library, db.library.Library)
        self.assertIsInstance(track.artist, db.artist.Artist)
        self.assertIsInstance(track.album, db.album.Album)
        self.assertIsInstance(track.disc, db.disc.Disc)
        self.assertIsInstance(track.decade, db.decade.Decade)
        self.assertIsInstance(track.year, db.year.Year)
        self.assertEqual(track.number, 1)
        self.assertEqual(track.playcount, 0)
        self.assertIsNone(track.lastplayed, None)
        self.assertEqual(track.length, 1.234)
        self.assertEqual(track.title, "Test Title")

    def test_genres(self):
        track = db.make_fake_track(1, 1.234, "Test Title", "/a/b/c/1.ogg")
        self.assertEqual(track.genres(), [ ])

        genre = db.genre.Table.find("Test Genre")
        genre.add_track(track)
        self.assertEqual(track.genres(), [ genre ])

        db.track.Table.delete(track)
        self.assertEqual(genre.get_n_tracks(), 0)

    def test_playlists(self):
        new = db.user.Table.lookup("New Tracks")
        favorites = db.user.Table.lookup("Favorites")

        track = db.make_fake_track(1, 1.234, "Test Title", "/a/b/c/1.ogg")
        self.assertEqual(track.playlists(), [ new ])
        favorites.add_track(track)
        self.assertEqual(track.playlists(), [ favorites, new ])

        db.track.Table.delete(track)
        self.assertEqual(new.get_n_tracks(), 0)
        self.assertEqual(favorites.get_n_tracks(), 0)

    def test_played(self):
        track = db.make_fake_track(1, 1.234, "Test Title", "/a/b/c/1.ogg")
        track.played()
        self.assertEqual(track.playcount, 1)
        self.assertEqual(track.lastplayed.date(), datetime.date.today())


class TestTrackTable(unittest.TestCase):
    def setUp(self):
        db.reset()

    def test_init(self):
        table = db.track.TrackTable()
        self.assertIsInstance(table, db.table.Table)
        self.assertEqual(table.table, "tracks")

        self.assertIsInstance(db.track.Table, db.track.TrackTable)
        db.sql.execute("SELECT trackid,libraryid,artistid,albumid,discid,decadeid,yearid FROM tracks")
        db.sql.execute("SELECT number,playcount,lastplayed,length,title,path FROM tracks")

    def test_insert(self):
        library = db.library.Table.find(pathlib.Path("/a/b/c"))
        artist  = db.artist.Table.find("Test Artist", "test artist")
        album   = artist.find_album("Test Album", datetime.date(2021, 3, 18))
        disc    = album.find_disc(1, None)
        decade  = db.decade.Table.find(2020)
        year    = decade.find_year(2021)
        track   = db.track.Table.insert(library, artist, album, disc, decade,
                                        year, 1, 1.234, "Test Title",
                                        pathlib.Path("/a/b/c/d.efg"))

        self.assertIsInstance(track, db.track.Track)

        with self.assertRaises(sqlite3.IntegrityError):
            db.track.Table.insert(library, artist, album, disc, decade, year,
                            1, 1.234, "Test Title", pathlib.Path("/a/b/c/d.efg"))

    def test_lookup(self):
        track = db.make_fake_track(1, 1.234, "Test Title", "/a/b/c/d.efg")
        self.assertEqual(db.track.Table.lookup(pathlib.Path("/a/b/c/d.efg")), track)
        self.assertIsNone(db.library.Table.lookup(pathlib.Path("/a/b/d/h.ijk")))

    def test_find(self):
        with self.assertRaises(NotImplementedError):
            db.track.Table.find(pathlib.Path("/a/b/c/d.efg"))
